@@ -6,6 +6,7 @@
     Copyright 2009-2011, Alan Calvert
     Copyright 2017-2019, Will Godfrey
     Copyright 2020 Kristian Amlie
+    Copyright 2023 Will Godfrey and others
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU General Public
@@ -34,9 +35,9 @@
 using func::power;
 
 
-LFOParams::LFOParams(float Pfreq_, unsigned char Pintensity_,
-                     unsigned char Pstartphase_, unsigned char PLFOtype_,
-                     unsigned char Prandomness_, unsigned char Pdelay_,
+LFOParams::LFOParams(float Pfreq_, float Pintensity_,
+                     float Pstartphase_, unsigned char PLFOtype_,
+                     float Prandomness_, float Pdelay_,
                      unsigned char Pcontinous_, int fel_, SynthEngine *_synth) :
     Presets(_synth),
     fel(fel_),
@@ -74,9 +75,9 @@ void LFOParams::defaults(void)
     Prandomness = Drandomness;
     Pdelay = Ddelay;
     Pcontinous = Dcontinous;
-    Pbpm = 0;
-    Pfreqrand = 0;
-    Pstretch = 64;
+    Pbpm = LFOSWITCH::BPM;
+    Pfreqrand = LFODEF::freqRnd.def;
+    Pstretch = LFODEF::stretch.def;
 }
 
 
@@ -98,13 +99,13 @@ void LFOParams::add2XML(XMLwrapper *xml)
         freqF = func::quantizedLFOfreqBPM(freqF);
     xml->addpar("freqI", freqF * float(Fmul2I));
     xml->addparreal("freq", freqF);
-    xml->addpar("intensity", Pintensity);
-    xml->addpar("start_phase", Pstartphase);
+    xml->addparcombi("intensity", Pintensity);
+    xml->addparcombi("start_phase", Pstartphase);
     xml->addpar("lfo_type", PLFOtype);
-    xml->addpar("randomness_amplitude", Prandomness);
-    xml->addpar("randomness_frequency", Pfreqrand);
-    xml->addpar("delay", Pdelay);
-    xml->addpar("stretch", Pstretch);
+    xml->addparcombi("randomness_amplitude", Prandomness);
+    xml->addparcombi("randomness_frequency", Pfreqrand);
+    xml->addparcombi("delay", Pdelay);
+    xml->addparcombi("stretch", Pstretch);
     xml->addparbool("continous",    Pcontinous);
     xml->addparbool("bpm", Pbpm);
 }
@@ -117,13 +118,13 @@ void LFOParams::getfromXML(XMLwrapper *xml)
     PfreqI = xml->getparreal("freq", Pfreq, 0.0, 1.0) * float(Fmul2I);
     setPfreq(PfreqI);
 
-    Pintensity = xml->getpar127("intensity", Pintensity);
-    Pstartphase = xml->getpar127("start_phase", Pstartphase);
+    Pintensity = xml->getparcombi("intensity", Pintensity,0,127);
+    Pstartphase = xml->getparcombi("start_phase", Pstartphase,0,127);
     PLFOtype = xml->getpar127("lfo_type", PLFOtype);
-    Prandomness = xml->getpar127("randomness_amplitude", Prandomness);
-    Pfreqrand = xml->getpar127("randomness_frequency", Pfreqrand);
-    Pdelay = xml->getpar127("delay", Pdelay);
-    Pstretch = xml->getpar127("stretch", Pstretch);
+    Prandomness = xml->getparcombi("randomness_amplitude", Prandomness,0,127);
+    Pfreqrand = xml->getparcombi("randomness_frequency", Pfreqrand,0,127);
+    Pdelay = xml->getparcombi("delay", Pdelay,0,127);
+    Pstretch = xml->getparcombi("stretch", Pstretch,0,127);
     Pcontinous = xml->getparbool("continous", Pcontinous);
     Pbpm = xml->getparbool("bpm", Pbpm);
     presetsUpdated();
@@ -137,7 +138,7 @@ float LFOlimit::getLFOlimits(CommandBlock *getData)
     int engine = getData->data.engine;
     int insertType = getData->data.parameter;
 
-    unsigned char type =0;
+    unsigned char type = 0;
 
     // LFO defaults
     int min = 0;
@@ -156,67 +157,83 @@ float LFOlimit::getLFOlimits(CommandBlock *getData)
             {
                 case TOPLEVEL::insertType::amplitude:
                     if (engine >= PART::engine::addVoice1)
-                        def = 0.708000;
+                        def = LFODEF::voiceAmpFreq.def;
                     else
-                        def = 0.62999f;
+                        def = LFODEF::ampFreq.def;
                     break;
                 case TOPLEVEL::insertType::frequency:
                     if (engine >= PART::engine::addVoice1)
-                        def = 0.393000f;
+                        def = LFODEF::voiceFreqFreq.def;
                     else
-                        def = 0.550999f;
+                        def = LFODEF::freqFreq.def;
                     break;
                 case TOPLEVEL::insertType::filter:
                     if (engine >= PART::engine::addVoice1)
-                        def = 0.393000f;
+                        def = LFODEF::voiceFiltFreq.def;
                     else
-                        def = 0.62999f;
+                        def = LFODEF::filtFreq.def;
                     break;
             }
             break;
         case LFOINSERT::control::depth:
+        type &= ~TOPLEVEL::type::Integer;
             if (engine >= PART::engine::addVoice1)
             {
                 switch(insertType)
                 {
                     case TOPLEVEL::insertType::amplitude:
-                        def = 32;
+                        def = LFODEF::voiceAmpDepth.def;
                         break;
                     case TOPLEVEL::insertType::frequency:
-                        def = 40;
+                        def = LFODEF::voiceFreqDepth.def;
                         break;
                     case TOPLEVEL::insertType::filter:
-                        def = 20;
+                        def = LFODEF::voiceFiltDepth.def;
+                        break;
+                    default:
+                        def = LFODEF::depth.def; // is this ever used?
                         break;
                 }
             }
             break;
         case LFOINSERT::control::delay:
+            type &= ~TOPLEVEL::type::Integer;
             if (engine >= PART::engine::addVoice1 && insertType == TOPLEVEL::insertType::amplitude)
-                def = 30;
+                def = LFODEF::voiceAmpDelay.def;
+            else
+                def = LFODEF::delay.def;
             break;
         case LFOINSERT::control::start:
+            type &= ~TOPLEVEL::type::Integer;
             if (engine < PART::engine::addVoice1 || insertType != TOPLEVEL::insertType::frequency)
-                def = 64;
+                def = LFODEF::start.def;
             break;
         case LFOINSERT::control::amplitudeRandomness:
+            type &= ~TOPLEVEL::type::Integer;
+            def = LFODEF::ampRnd.def;
             break;
         case LFOINSERT::control::type:
-            max = 9;
+            max = LFODEF::type.max;
+            def = LFODEF::type.def;
             type &= ~learnable;
             break;
         case LFOINSERT::control::continuous:
-            max = 1;
+            max = true;
+            def = LFOSWITCH::continuous;
             type &= ~learnable;
             break;
         case LFOINSERT::control::bpm:
-            max = 1;
+            max = true;
+            def = LFOSWITCH::BPM;
             type &= ~learnable;
             break;
         case LFOINSERT::control::frequencyRandomness:
+            type &= ~TOPLEVEL::type::Integer;
+            def = LFODEF::freqRnd.def;
             break;
         case LFOINSERT::control::stretch:
-            def = 64;
+            type &= ~TOPLEVEL::type::Integer;
+            def = LFODEF::stretch.def;
             break;
 
         default:
